@@ -6,6 +6,7 @@
 
 import json
 import os
+import sys
 from os.path import join as pjoin
 
 from jupyter_core.application import JupyterApp, NoStart, base_aliases, base_flags
@@ -20,7 +21,7 @@ from jupyterlab_server import (
     WorkspaceListApp,
 )
 from notebook_shim.shim import NotebookConfigShimMixin as NBClassicConfigShimMixin
-from traitlets import Bool, Instance, Unicode, default
+from traitlets import Bool, Instance, Type, Unicode, default
 
 from ._version import __version__
 from .commands import (
@@ -42,6 +43,14 @@ from .commands import (
 )
 from .coreconfig import CoreConfig
 from .debuglog import DebugLogFileMixin
+from .handlers.announcements import (
+    CheckForUpdate,
+    CheckForUpdateABC,
+    CheckForUpdateHandler,
+    NewsHandler,
+    check_update_handler_path,
+    news_handler_path,
+)
 from .handlers.build_handler import Builder, BuildHandler, build_path
 from .handlers.error_handler import ErrorHandler
 from .handlers.extension_manager_handler import (
@@ -49,7 +58,6 @@ from .handlers.extension_manager_handler import (
     ExtensionManager,
     extensions_handler_path,
 )
-from .handlers.yjs_echo_ws import YjsEchoWebSocket
 
 DEV_NOTE = """You're running JupyterLab from source.
 If you're working on the TypeScript sources of JupyterLab, try running
@@ -464,7 +472,13 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
     )
     flags["collaborative"] = (
         {"LabApp": {"collaborative": True}},
-        "Whether to enable collaborative mode (experimental).",
+        "Whether to enable collaborative mode.",
+    )
+    flags["future-skip-styles-for-disabled"] = (
+        {"LabApp": {"future_skip_styles_for_disabled": True}},
+        """Whether to skip loading styles for disabled prebuilt extensions.
+        This will be the default behavior starting with JupyterLab 4.0
+        (and this flag will be removed).""",
     )
 
     subcommands = dict(
@@ -536,8 +550,28 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         help="Whether to expose the global app instance to browser via window.jupyterlab",
     )
 
-    collaborative = Bool(
-        False, config=True, help="Whether to enable collaborative mode (experimental)."
+    future_skip_styles_for_disabled = Bool(
+        False,
+        config=True,
+        help="""Whether to skip loading styles for disabled prebuilt extensions.
+        This will be the default behavior starting with JupyterLab 4.0
+        (and this flag will be removed).""",
+    )
+
+    collaborative = Bool(False, config=True, help="Whether to enable collaborative mode.")
+
+    news_url = Unicode(
+        "https://jupyterlab.github.io/assets/feed.xml",
+        allow_none=True,
+        help="""URL that serves news Atom feed; by default the JupyterLab organization announcements will be fetched. Set to None to turn off fetching announcements.""",
+        config=True,
+    )
+
+    check_for_updates_class = Type(
+        default_value=CheckForUpdate,
+        klass=CheckForUpdateABC,
+        config=True,
+        help="""A callable class that receives the current version at instantiation and calling it must return asynchronously a string indicating which version is available and how to install or None if no update is available. The string supports Markdown format.""",
     )
 
     @default("app_dir")
@@ -646,6 +680,7 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         page_config["quitButton"] = self.serverapp.quit_button
         page_config["collaborative"] = self.collaborative
         page_config["allow_hidden_files"] = self.serverapp.contents_manager.allow_hidden
+        page_config["futureSkipStylesForDisabled"] = self.future_skip_styles_for_disabled
 
         # Client-side code assumes notebookVersion is a JSON-encoded string
         page_config["notebookVersion"] = json.dumps(jpserver_version_info)
@@ -663,10 +698,6 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         build_handler = (build_path, BuildHandler, {"builder": builder})
         handlers.append(build_handler)
 
-        # Yjs Echo WebSocket handler
-        yjs_echo_handler = (r"/api/yjs/(.*)", YjsEchoWebSocket)
-        handlers.append(yjs_echo_handler)
-
         errored = False
 
         if self.core_mode:
@@ -700,6 +731,27 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
             ext_handler = (extensions_handler_path, ExtensionHandler, {"manager": ext_manager})
             handlers.append(ext_handler)
 
+            # Add announcement handlers
+            page_config["news"] = {"disabled": self.news_url is None}
+            handlers.extend(
+                [
+                    (
+                        check_update_handler_path,
+                        CheckForUpdateHandler,
+                        {
+                            "update_checker": self.check_for_updates_class(__version__),
+                        },
+                    ),
+                    (
+                        news_handler_path,
+                        NewsHandler,
+                        {
+                            "news_url": self.news_url,
+                        },
+                    ),
+                ]
+            )
+
         # If running under JupyterHub, add more metadata.
         if "hub_prefix" in self.serverapp.tornado_settings:
             tornado_settings = self.serverapp.tornado_settings
@@ -711,8 +763,11 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
             # Assume the server_name property indicates running JupyterHub 1.0.
             if hasattr(self.serverapp, "server_name"):
                 page_config["hubServerName"] = self.serverapp.server_name
-            api_token = os.getenv("JUPYTERHUB_API_TOKEN", "")
-            page_config["token"] = api_token
+            # avoid setting API token in page config
+            # $JUPYTERHUB_API_TOKEN identifies the server, not the client
+            # but at least make sure we don't use the token
+            # if the serverapp set one
+            page_config["token"] = ""
 
         # Update Jupyter Server's webapp settings with jupyterlab settings.
         self.serverapp.web_app.settings["page_config_data"] = page_config
@@ -725,6 +780,16 @@ class LabApp(NBClassicConfigShimMixin, LabServerApp):
         """Subclass because the ExtensionApp.initialize() method does not take arguments"""
         super().initialize()
 
+        if self.collaborative and jpserver_version_info < (2, 0, 0):
+            jpserver_version = ".".join(filter(lambda p: len(p) > 0, map(str, jpserver_version_info)))
+            self.log.critical(f"""To enable real-time collaboration, you must install Jupyter Server v2 (current version is {jpserver_version}).
+
+You can install it using pip for example:
+
+  python -m pip install "jupyter_server>=2.0.0"
+""")
+            sys.exit(1)
+
 
 # -----------------------------------------------------------------------------
 # Main entry point
